'''
Handle the conversion of the Planet object to the params list which is
required by the C code.

Required arguments are:
    1) G in units of AU^3/day^2/M_sun
    2) MStar in units of M_sun
    3-9) Planetary parameters:
        * Mplanet in units of M_sun
        * Period in days
        * E between 0 and 1
        * I in units of degrees
        * Longnode in units of degrees
        * Argument in units of degrees
        * mean anomaly in units of degrees
'''


def planets_to_params(stellar_mass, planets, G=0.000295994511):
    out = [G, stellar_mass]
    for planet in planets:
        out.extend([getattr(planet, key) for key in Planet.KEYS])
    return out


class Planet(object):

    '''
    Planet class to define each planet in the system.
    '''

    KEYS = ['mass', 'period', 'eccentricity', 'inclination',
            'longnode', 'argument', 'mean_anomaly']

    def __init__(self, mass, period, eccentricity, inclination,
                 longnode, argument, mean_anomaly):
        '''
        Construct a planet. Required arguments are:
            * mass: Mplanet in units of M_sun
            * period: Period in days
            * eccentricity: E between 0 and 1
            * inclination: I in units of degrees
            * longnode: Longnode in units of degrees
            * argument: Argument in units of degrees
            * mean_anomaly: mean anomaly in units of degrees
        '''
        self.mass = mass
        self.period = period
        self.eccentricity = eccentricity
        self.inclination = inclination
        self.longnode = longnode
        self.argument = argument
        self.mean_anomaly = mean_anomaly



# -*- coding: utf-8 -*-


from setuptools import setup, Extension
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

ttvfast = Extension("ttvfast._ttvfast",
            sources=["src/ttvfast_wrapper.c",
                "external/TTVFast/c_version/TTVFast.c"],
            include_dirs=['external/TTVFast/c_version'],
            extra_compile_args=['-std=c99'],
            # Debug mode
            # define_macros=[('DEBUG', True)],
            )

with open(path.join(here, 'README.rst'), encoding='utf-8') as infile:
    long_description = infile.read()

setup(
    name='ttvfast',
    version='0.2.0',
    description='Python wrapper to ttvfast',
    url='https://github.com/mindriot101/ttvfast-python',
    long_description=long_description,
    author='Simon Walker',
    author_email='s.r.walker101@googlemail.com',
    license='GPL',
    packages=['ttvfast', ],
    ext_modules=[ttvfast, ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Topic :: Scientific/Engineering :: Physics',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



# -*- coding: utf-8 -*-

"Fast TTV computation"

from ._ttvfast import _ttvfast as _ttvfast_fn
from . import models


__all__ = ['ttvfast']


def ttvfast(planets, stellar_mass, time, dt, total, rv_times=None, input_flag=0):
    '''
    Run the TTV fast function. See https://github.com/kdeck/TTVFast.
    Program arguments:
    * planets: list of `models.Planet` instances
    * stellar_mass: stellar mass (solar masses)
    * time: start point for integration (days)
    * dt: time step for the integration (days)
    * total: end point for integration (days)
    * rv_times: rv measurement times
    * input_flag: input coordinate system (0, 1 or 2)

    The `input_flag` argument corresponds to:
        0 = Jacobi
        1 = astrocentric elements
        2 = astrocentric cartesian
    '''
    if input_flag not in [0, 1, 2]:
        raise ValueError('Invalid `input_flag` value. Must be 0, 1 or 2')

    params = models.planets_to_params(stellar_mass, planets)
    n_plan = len(planets)

    len_rv = len(rv_times) if rv_times is not None else 0
    positions, rv = _ttvfast_fn(
        params, dt, time, total, n_plan, input_flag, len_rv, rv_times)
    return {'positions': positions, 'rv': rv}

__all__ = ['ttvfast']
